---
id: "2019-03-21-10-38"
date: "2019/03/21 10:38:00"
title: "springboot超级详细的日志配置(基于logback)"
tags: ["java", "springboot", "logback", "log"]
categories:
  - "java"
  - "spring boot学习"
---

![嘿](https://raw.githubusercontent.com/FleyX/files/master/blogImg/20190320135049.png)

# 前言

&emsp;&emsp;java web 下有好几种日志框架，比如：logback，log4j，log4j2（slj4f 并不是一种日志框架，它相当于定义了规范，实现了这个规范的日志框架就能够用 slj4f 调用）。其中性能最高的应该使 logback 了，而且 springboot 默认使用的也是 logback 日志，所以本篇将会详细的讲解 logback 的日志配置方案。

本篇主要内容如下：

- logback 配置文件的构成
- 如何将日志输出到文件
- 如何按时间，按大小切分日志
- 如何将让一个日志文件中只有一种日志级别
- 如何将指定包下的日志输出到指定的日志文件中

# 简单使用

&emsp;&emsp;如果无需复杂的日志配置，执行简单设置日志打印级别，打印方式可直接再 application.yml 中配置。

&emsp;&emsp;默认情况下 Spring Boot 将 info 级别的日志输出到控制台中，不会写到日志文件，且不能进行复杂配置。

<!-- more -->

## 打印到文件中

&emsp;&emsp;想要将日志输出到文件中，可通过如下两个配置：

```yaml
logging:
  # 配置输出额日志文件名,可以带路径
  #  file: out.log
  # 配置日志存放路径,日志文件名为：spring.log
  path: ./log
  file:
    # 设置日志文件大小
    max-size: 10MB
```

**注意**:file 和 path 是不能同时配置的，如果同时配置`path`不会生效。

## 打印级别控制

&emsp;&emsp;可通过配置控制日志打印级别，格式如下：

```yaml
logging.level.*=TRACE/DEBUG/INFO/...
```

\*可以为包名或 Logger 名，如下：

```yaml
logging:
  level:
    # root日志以WARN级别输出
    root: info
    # 此包下所有class以DEBUG级别输出
    com.example.log_demo.log1: warn
```

# logback 详细配置

&emsp;&emsp;接下来说明如何通过独立的 xml 配置文件来配置日志打印。虽然 springboot 是要消灭 xml 的，但是有些复杂功能还是得编写 xml。使用 xml 后要将 application.yml 中的配置去掉，避免冲突.

&emsp;&emsp;根据不同的日志系统，按照指定的规则组织配置文件名，并放在 resources 目录下，就能自动被 spring boot 加载：

- Logback：logback-spring.xml, logback-spring.groovy, logback.xml, logback.groovy
- Log4j: log4j-spring.properties, log4j-spring.xml, log4j.properties, log4j.xml
- Log4j2: log4j2-spring.xml, log4j2.xml
- JDK (Java Util Logging)： logging.properties

想要自定义文件名的可配置：`logging.config`指定配置文件名:

```properties
logging.config=classpath:logging-config.xml
```

Spring Boot 官方推荐优先使用带有 -spring 的文件名作为你的日志配置（如使用 logback-spring.xml ，而不是 logback.xml ），命名为 logback-spring.xml 的日志配置文件， spring boot 可以为它添加一些 spring boot 特有的配置项（下面会提到）。

## logback 配置文件的组成

&emsp;&emsp;根节点`<configuration>`有 5 个子节点，下面来进行一一介绍。

### `<root>`节点

&emsp;&emsp;root 节点是必选节点，用来指定最基础的日志输出级别，只有一个 level 属性，用于设置打印级别，可选如下：`TRACE`,`DEBUG`,`INFO`,`WARN`,`ERROR`,`ALL`,`OFF`。

&emsp;&emsp;root 节点可以包含 0 个或多个元素，将`appender`添加进来。如下：

```xml
<root level="debug">
 <appender-ref ref="console" />
 <appender-ref ref="file" />
</root>
```

appender 也是子节点之一，将会在后面说明。

### `<contextName>`节点

&emsp;&emsp;设置上下文名称，默认为`default`，可通过`%contextName`来打印上下文名称，一般不使用此属性。

### `<property>`节点

&emsp;&emsp;用于定义变量，方便使用。有两个属性：name,value。定义变量后，可以使用`${}`来使用变量。如下：

```xml
<property name="path" value="./log"/>
<property name="appname" value="app"/>
```

### `<appender>`节点

&emsp;&emsp;appender 用来格式化日志输出的节点，这个最重要。有两个属性：

- name:该本 appender 命名
- class:指定输出策略，通常有两种：控制台输出，文件输出

下面通过例子来说明这个怎么用：

1. 输出到控制台/按时间输出日志

```xml
<?xml version="1.0" encoding="UTF-8"?>
<configuration debug="false">
    <!--设置存储路径变量-->
    <property name="LOG_HOME" value="./log"/>

    <!--控制台输出appender-->
    <appender name="console" class="ch.qos.logback.core.ConsoleAppender">
        <!--设置输出格式-->
        <encoder class="ch.qos.logback.classic.encoder.PatternLayoutEncoder">
            <!--格式化输出：%d表示日期，%thread表示线程名，%-5level：级别从左显示5个字符宽度%msg：日志消息，%n是换行符-->
            <pattern>%d{yyyy-MM-dd HH:mm:ss.SSS} [%thread] %-5level %logger{50} - %msg%n</pattern>
            <!--设置编码-->
            <charset>UTF-8</charset>
        </encoder>
    </appender>

    <!--文件输出,时间窗口滚动-->
    <appender name="timeFileOutput" class="ch.qos.logback.core.rolling.RollingFileAppender">
        <!--日志名,指定最新的文件名，其他文件名使用FileNamePattern -->
        <File>${LOG_HOME}/timeFile/out.log</File>
        <!--文件滚动模式-->
        <rollingPolicy class="ch.qos.logback.core.rolling.TimeBasedRollingPolicy">
            <!--日志文件输出的文件名,可设置文件类型为gz,开启文件压缩-->
            <FileNamePattern>${LOG_HOME}/timeFile/info.%d{yyyy-MM-dd}.%i.log.gz</FileNamePattern>
            <!--日志文件保留天数-->
            <MaxHistory>30</MaxHistory>
            <!--按大小分割同一天的-->
            <timeBasedFileNamingAndTriggeringPolicy class="ch.qos.logback.core.rolling.SizeAndTimeBasedFNATP">
                <maxFileSize>10MB</maxFileSize>
            </timeBasedFileNamingAndTriggeringPolicy>
        </rollingPolicy>

        <!--输出格式-->
        <encoder class="ch.qos.logback.classic.encoder.PatternLayoutEncoder">
            <!--格式化输出：%d表示日期，%thread表示线程名，%-5level：级别从左显示5个字符宽度%msg：日志消息，%n是换行符-->
            <pattern>%d{yyyy-MM-dd HH:mm:ss.SSS} [%thread] %-5level %logger{50} - %msg%n</pattern>
            <!--设置编码-->
            <charset>UTF-8</charset>
        </encoder>

    </appender>

    <!--指定基础的日志输出级别-->
    <root level="INFO">
        <!--appender将会添加到这个loger-->
        <appender-ref ref="console"/>
        <appender-ref ref="timeFileOutput"/>
    </root>
</configuration>
```

2. 设置只输出单个级别

&emsp;&emsp;在 appender 中设置,filter 子节点，在默认级别上再此过滤，配置 onMatch，onMismatch 可实现只输出单个级别

```xml
<appender ...>
    <filter class="ch.qos.logback.classic.filter.LevelFilter">
        <level>INFO</level>
        <!--接受匹配-->
        <onMatch>ACCEPT</onMatch>
        <!--拒绝不匹配的-->
        <onMismatch>DENY</onMismatch>
    </filter>
</appender>
```

### `<logger>`节点

#### 构成

&emsp;&emsp;此节点用来设置一个包或具体的某一个类的日志打印级别、以及指定`<appender>`，有以下三个属性：

- name: 必须。用来指定受此 loger 约束的某个包或者某个具体的类
- level：可选。设置打印级别。默认为 root 的级别。
- addtivity: 可选。是否向上级 loger(也就是 root 节点)传递打印信息。默认为 true。

使用示例如下：

1. 不指定级别，不指定 appender

```xml
<!-- 控制com.example.service下类的打印，使用root的level和appender -->
<logger name="com.example.service"/>
```

2.指定级别，不指定 appender

```xml
<!-- 控制com.example.service下类的打印，使用root的appender打印warn级别日志 -->
<logger name="com.example.service" level="WARN"/>
```

3.指定级别，指定 appender

```xml
<!-- 控制com.example.service下类的打印，使用console打印warn级别日志 -->
<!-- 设置addtivity是因为这里已经指定了appender，如果再向上传递就会被root下的appender再次打印 -->
<logger name="com.example.service" level="WARN" addtivity="false">
    <appender-ref ref="console">
</logger>
```

通过指定 appender 就能将指定的包下的日志打印到指定的文件中。

#### 多环境日志输出

&emsp;&emsp;通过设置文件名为-spring 结尾，可分环境配置 logger，示例如下：

```xml
<configuration>
    <!-- 测试环境+开发环境. 多个使用逗号隔开. -->
    <springProfile name="test,dev">
        <logger name="com.example.demo.controller" level="DEBUG" additivity="false">
            <appender-ref ref="console"/>
        </logger>
    </springProfile>
    <!-- 生产环境. -->
    <springProfile name="prod">
        <logger name="com.example.demo" level="INFO" additivity="false">
            <appender-ref ref="timeFileOutput"/>
        </logger>
    </springProfile>
</configuration>
```

通过配置`spring.profiles.active`也能做到切换上面的 logger 打印设置

# 结束

&emsp;&emsp;日志打印可以很复杂， 这里只是说明了常用的 logback 配置，代码详见：[github](https://github.com/FleyX/demo-project/tree/master/spring-boot/log-demo)

**本文原创发布于：**[http://www.tapme.top/blog/detail/2019-03-21-10-38](http://www.tapme.top/blog/detail/2019-03-20-10-38)
